@@ -15,67 +15,58 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: $ */
 
 package org.apache.fop.apps;
 
 import java.awt.color.ColorSpace;
-import java.awt.color.ICC_ColorSpace;
-import java.awt.color.ICC_Profile;
 import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.net.MalformedURLException;
+import java.net.URL;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.Map;
 import java.util.Set;
 
 import javax.xml.transform.Source;
 import javax.xml.transform.TransformerException;
 import javax.xml.transform.URIResolver;
-import javax.xml.transform.stream.StreamSource;
 
 import org.xml.sax.SAXException;
 
 import org.apache.avalon.framework.configuration.Configuration;
-import org.apache.avalon.framework.configuration.ConfigurationException;
-import org.apache.avalon.framework.configuration.DefaultConfigurationBuilder;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
 import org.apache.fop.fo.ElementMapping;
 import org.apache.fop.fo.ElementMappingRegistry;
+import org.apache.fop.fonts.FontCache;
 import org.apache.fop.hyphenation.HyphenationTreeResolver;
 import org.apache.fop.image.ImageFactory;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
 import org.apache.fop.render.RendererFactory;
 import org.apache.fop.render.XMLHandlerRegistry;
+import org.apache.fop.util.ColorSpaceCache;
 import org.apache.fop.util.ContentHandlerFactoryRegistry;
 
 /**
- * Factory class which instantiates new Fop and FOUserAgent instances. This class also holds
- * environmental information and configuration used by FOP. Information that may potentially be
- * different for each rendering run can be found and managed in the FOUserAgent.
+ * Factory class which instantiates new Fop and FOUserAgent instances. This
+ * class also holds environmental information and configuration used by FOP.
+ * Information that may potentially be different for each rendering run can be
+ * found and managed in the FOUserAgent.
  */
 public class FopFactory {
-
-    /** Defines the default source resolution (72dpi) for FOP */
-    private static final float DEFAULT_SOURCE_RESOLUTION = 72.0f; //dpi
-    /** Defines the default page-height */
-    private static final String DEFAULT_PAGE_HEIGHT = "11in";
-    /** Defines the default page-width */
-    private static final String DEFAULT_PAGE_WIDTH = "8.26in";
-
+    
     /** logger instance */
     private static Log log = LogFactory.getLog(FopFactory.class);
     
     /** Factory for Renderers and FOEventHandlers */
-    private RendererFactory rendererFactory = new RendererFactory();
+    private RendererFactory rendererFactory;
     
     /** Registry for XML handlers */
-    private XMLHandlerRegistry xmlHandlers = new XMLHandlerRegistry();
+    private XMLHandlerRegistry xmlHandlers;
     
     /** The registry for ElementMapping instances */
     private ElementMappingRegistry elementMappingRegistry;
@@ -84,56 +75,86 @@ public class FopFactory {
     private ContentHandlerFactoryRegistry contentHandlerFactoryRegistry 
                 = new ContentHandlerFactoryRegistry();
     
-    /** Our default resolver if none is set */
-    private URIResolver foURIResolver = new FOURIResolver();
-    /** A user settable URI Resolver */
-    private URIResolver uriResolver = null;
-
     /** The resolver for user-supplied hyphenation patterns */
-    private HyphenationTreeResolver hyphResolver;
-    
-    private ImageFactory imageFactory = new ImageFactory();
+    private HyphenationTreeResolver hyphResolver = null;
 
-    /** user configuration */
-    private Configuration userConfig = null;
-
-    /** The base URL for all font URL resolutions */
-    private String fontBaseURL;
+    private ColorSpaceCache colorSpaceCache = null;
     
+    /** Image factory for creating fop image objects */
+    private ImageFactory imageFactory;
+
+    /** Configuration layer used to configure fop */
+    private FopFactoryConfigurator config = null;
+        
+    /**
+     *  The base URL for all URL resolutions, especially for
+     *  external-graphics.
+     */
+    private String base = null;
+
+    /** The base URL for all font URL resolutions. */
+    private String fontBase = null;
+
+    /** The base URL for all hyphen URL resolutions. */
+    private String hyphenBase = null;
+
     /**
      * FOP has the ability, for some FO's, to continue processing even if the
      * input XSL violates that FO's content model.  This is the default  
      * behavior for FOP.  However, this flag, if set, provides the user the
-     * ability for FOP to halt on all content model violations if desired.   
+     * ability for FOP to halt on all content model violations if desired.
      */ 
-    private boolean strictValidation = true;
+    private boolean strictFOValidation = FopFactoryConfigurator.DEFAULT_STRICT_FO_VALIDATION;
+
+    /**
+     * FOP will validate the contents of the user configuration strictly
+     * (e.g. base-urls and font urls/paths).
+     */
+    private boolean strictUserConfigValidation
+        = FopFactoryConfigurator.DEFAULT_STRICT_USERCONFIG_VALIDATION;
+
+    /** Font cache to speed up auto-font configuration (null if disabled) */
+    private FontCache fontCache = null;
 
     /** Allows enabling kerning on the base 14 fonts, default is false */
     private boolean enableBase14Kerning = false;
     
     /** Source resolution in dpi */
-    private float sourceResolution = DEFAULT_SOURCE_RESOLUTION;
-    private String pageHeight = DEFAULT_PAGE_HEIGHT;
-    private String pageWidth = DEFAULT_PAGE_WIDTH;
+    private float sourceResolution = FopFactoryConfigurator.DEFAULT_SOURCE_RESOLUTION;
+
+    /** Target resolution in dpi */
+    private float targetResolution = FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION;
+
+    /** Page height */
+    private String pageHeight = FopFactoryConfigurator.DEFAULT_PAGE_HEIGHT;
+    
+    /** Page width */
+    private String pageWidth = FopFactoryConfigurator.DEFAULT_PAGE_WIDTH;
 
     /** @see #setBreakIndentInheritanceOnReferenceAreaBoundary(boolean) */
-    private boolean breakIndentInheritanceOnReferenceAreaBoundary = false;
+    private boolean breakIndentInheritanceOnReferenceAreaBoundary
+        = FopFactoryConfigurator.DEFAULT_BREAK_INDENT_INHERITANCE;
 
     /** Optional overriding LayoutManagerMaker */
     private LayoutManagerMaker lmMakerOverride = null;
 
-    private Set ignoredNamespaces = new java.util.HashSet();
-    
-    /** Map with cached ICC based ColorSpace objects. */
-    private Map colorSpaceMap = null;
+    private Set ignoredNamespaces;
+
+    private FOURIResolver foURIResolver;
     
     /**
      * Main constructor.
      */
     protected FopFactory() {
+        this.config = new FopFactoryConfigurator(this);
         this.elementMappingRegistry = new ElementMappingRegistry(this);
-        // Use a synchronized Map - I am not really sure this is needed, but better safe than sorry.
-        this.colorSpaceMap = Collections.synchronizedMap(new java.util.HashMap());
+        this.foURIResolver = new FOURIResolver(validateUserConfigStrictly());
+        this.colorSpaceCache = new ColorSpaceCache(foURIResolver);
+        this.imageFactory = new ImageFactory();        
+        this.rendererFactory = new RendererFactory();
+        this.xmlHandlers = new XMLHandlerRegistry();
+        this.ignoredNamespaces = new java.util.HashSet();
+        setUseCache(FopFactoryConfigurator.DEFAULT_USE_CACHE);
     }
     
     /**
@@ -149,6 +170,7 @@ public class FopFactory {
      * are particular to a rendering run. Don't reuse instances over multiple rendering runs but
      * instead create a new one each time and reuse the FopFactory.
      * @return the newly created FOUserAgent instance initialized with default values
+     * @throws FOPException 
      */
     public FOUserAgent newFOUserAgent() {
         FOUserAgent userAgent = new FOUserAgent(this);
@@ -296,25 +318,97 @@ public class FopFactory {
     }
 
     /**
-     * Sets the font base URL.
-     * @param fontBaseURL font base URL
+     * cleans the base url
+     * @param base
+     * @return
+     * @throws MalformedURLException
+     * @throws URISyntaxException 
      */
-    public void setFontBaseURL(String fontBaseURL) {
-        this.fontBaseURL = fontBaseURL;
+    private String checkBaseURL(String base) throws MalformedURLException {
+        if (!base.endsWith("/")) {
+            // The behavior described by RFC 3986 regarding resolution of relative
+            // references may be misleading for normal users:
+            // file://path/to/resources + myResource.res -> file://path/to/myResource.res
+            // file://path/to/resources/ + myResource.res -> file://path/to/resources/myResource.res
+            // We assume that even when the ending slash is missing, users have the second
+            // example in mind
+            base += "/";
+        }
+        File dir = new File(base);
+        try {
+            base = (dir.isDirectory() ? dir.toURL() : new URL(base)).toExternalForm(); 
+        } catch (MalformedURLException mfue) {
+            if (strictUserConfigValidation) {
+                throw mfue;
+            }
+            log.error(mfue.getMessage());
+        }
+        return base;
+    }
+    
+    /**
+     * Sets the base URL.
+     * @param base base URL
+     * @throws MalformedURLException 
+     * @throws URISyntaxException 
+     */
+    public void setBaseURL(String base) throws MalformedURLException {
+        this.base = checkBaseURL(base);
+    }
+
+    /**
+     * Returns the base URL.
+     * @return the base URL
+     */
+    public String getBaseURL() {
+        return this.base;
+    }
+    
+    /**
+     * Sets the font base URL.
+     * @param fontBase font base URL
+     * @throws MalformedURLException 
+     * @throws URISyntaxException 
+     */
+    public void setFontBaseURL(String fontBase) throws MalformedURLException {
+        this.fontBase = checkBaseURL(fontBase);
     }
 
     /** @return the font base URL */
     public String getFontBaseURL() {
-        return this.fontBaseURL;
+        return this.fontBase;
     }
 
+    /** @return the hyphen base URL */
+    public String getHyphenBaseURL() {
+        return this.hyphenBase;
+    }
+
+    /**
+     * Sets the hyphen base URL.
+     * @param hyphenBase hythen base URL
+     * @throws MalformedURLException 
+     * @throws URISyntaxException 
+     * */
+    public void setHyphenBaseURL(final String hyphenBase) throws MalformedURLException {
+        if (hyphenBase != null) {
+            setHyphenationTreeResolver(
+            new HyphenationTreeResolver() {
+                public Source resolve(String href) {
+                    return resolveURI(href, hyphenBase);
+                }
+            });
+        }
+        this.hyphenBase = checkBaseURL(hyphenBase);
+    }
+    
     /**
      * Sets the URI Resolver. It is used for resolving factory-level URIs like hyphenation
      * patterns and as backup for URI resolution performed during a rendering run. 
      * @param resolver the new URI resolver
      */
-    public void setURIResolver(URIResolver resolver) {
-        this.uriResolver = resolver;
+    public void setURIResolver(URIResolver uriResolver) {
+        foURIResolver.setCustomURIResolver(uriResolver);
     }
 
     /**
@@ -322,7 +416,7 @@ public class FopFactory {
      * @return the URI Resolver
      */
     public URIResolver getURIResolver() {
-        return this.uriResolver;
+        return foURIResolver;
     }
 
     /** @return the HyphenationTreeResolver for resolving user-supplied hyphenation patterns. */
@@ -330,13 +424,21 @@ public class FopFactory {
         return this.hyphResolver;
     }
     
+    /**
+     * sets the HyphenationTreeResolver
+     * @param hyphResolver
+     */
+    public void setHyphenationTreeResolver(HyphenationTreeResolver hyphResolver) {
+        this.hyphResolver = hyphResolver;
+    }
+
     /**
      * Activates strict XSL content model validation for FOP
      * Default is false (FOP will continue processing where it can)
      * @param validateStrictly true to turn on strict validation
      */
     public void setStrictValidation(boolean validateStrictly) {
-        this.strictValidation = validateStrictly;
+        this.strictFOValidation = validateStrictly;
     }
 
     /**
@@ -344,7 +446,7 @@ public class FopFactory {
      * @return true of strict validation turned on, false otherwise
      */
     public boolean validateStrictly() {
-        return strictValidation;
+        return strictFOValidation;
     }
 
     /**
@@ -399,14 +501,51 @@ public class FopFactory {
     public float getSourcePixelUnitToMillimeter() {
         return 25.4f / getSourceResolution(); 
     }
-    
+
+    /**
+     * Sets the source resolution in dpi. This value is used to interpret the pixel size
+     * of source documents like SVG images and bitmap images without resolution information.
+     * @param dpi resolution in dpi
+     */
+    public void setSourceResolution(float dpi) {
+        this.sourceResolution = dpi;
+        if (log.isDebugEnabled()) {
+            log.debug("source-resolution set to: " + sourceResolution 
+                    + "dpi (px2mm=" + getSourcePixelUnitToMillimeter() + ")");
+        }
+    }
+
+    /** @return the resolution for resolution-dependant output */
+    public float getTargetResolution() {
+        return this.targetResolution;
+    }
+
+    /**
+     * Returns the conversion factor from pixel units to millimeters. This
+     * depends on the desired target resolution.
+     * @return float conversion factor
+     * @see #getTargetResolution()
+     */
+    public float getTargetPixelUnitToMillimeter() {
+        return 25.4f / this.targetResolution; 
+    }
+
+    /**
+     * Sets the source resolution in dpi. This value is used to interpret the pixel size
+     * of source documents like SVG images and bitmap images without resolution information.
+     * @param dpi resolution in dpi
+     */
+    public void setTargetResolution(float dpi) {
+        this.targetResolution = dpi;
+    }
+
     /**
      * Sets the source resolution in dpi. This value is used to interpret the pixel size
      * of source documents like SVG images and bitmap images without resolution information.
      * @param dpi resolution in dpi
      */
     public void setSourceResolution(int dpi) {
-        this.sourceResolution = dpi;
+        setSourceResolution((float)dpi);
     }
     
     /**
@@ -427,6 +566,9 @@ public class FopFactory {
      */
     public void setPageHeight(String pageHeight) {
         this.pageHeight = pageHeight;
+        if (log.isDebugEnabled()) {
+            log.debug("Default page-height set to: " + pageHeight);
+        }
     }
     
     /**
@@ -447,6 +589,9 @@ public class FopFactory {
      */
     public void setPageWidth(String pageWidth) {
         this.pageWidth = pageWidth;
+        if (log.isDebugEnabled()) {
+            log.debug("Default page-width set to: " + pageWidth);
+        }
     }
     
     /**
@@ -482,7 +627,7 @@ public class FopFactory {
     public Set getIgnoredNamespace() {
         return Collections.unmodifiableSet(this.ignoredNamespaces);
     }
-    
+
     //------------------------------------------- Configuration stuff
     
     /**
@@ -491,46 +636,27 @@ public class FopFactory {
      * @throws IOException if an I/O error occurs
      * @throws SAXException if a parsing error occurs
      */
-    public void setUserConfig(File userConfigFile)
-                throws SAXException, IOException {
-        try {
-            DefaultConfigurationBuilder cfgBuilder = new DefaultConfigurationBuilder();
-            setUserConfig(cfgBuilder.buildFromFile(userConfigFile));
-        } catch (ConfigurationException cfge) {
-            log.error("Error loading configuration: "
-                    + cfge.getMessage());
-        }
+    public void setUserConfig(File userConfigFile) throws SAXException, IOException {
+        config.setUserConfig(userConfigFile);
     }
-    
+
     /**
      * Set the user configuration from an URI.
      * @param uri the URI to the configuration file
      * @throws IOException if an I/O error occurs
      * @throws SAXException if a parsing error occurs
      */
-    public void setUserConfig(String uri)
-                throws SAXException, IOException {
-        try {
-            DefaultConfigurationBuilder cfgBuilder = new DefaultConfigurationBuilder();
-            setUserConfig(cfgBuilder.build(uri));
-        } catch (ConfigurationException cfge) {
-            log.error("Error loading configuration: "
-                    + cfge.getMessage());
-        }
+    public void setUserConfig(String uri) throws SAXException, IOException {
+        config.setUserConfig(uri);
     }
     
     /**
      * Set the user configuration.
      * @param userConfig configuration
+     * @throws FOPException if a configuration problem occurs 
      */
-    public void setUserConfig(Configuration userConfig) {
-        this.userConfig = userConfig;
-        try {
-            initUserConfig();
-        } catch (ConfigurationException cfge) {
-            log.error("Error initializing factory configuration: "
-                    + cfge.getMessage());
-        }
+    public void setUserConfig(Configuration userConfig) throws FOPException {
+        config.setUserConfig(userConfig);
     }
 
     /**
@@ -538,107 +664,71 @@ public class FopFactory {
      * @return the user configuration
      */
     public Configuration getUserConfig() {
-        return userConfig;
+        return config.getUserConfig();
+    }
+
+    /**
+     * Is the user configuration to be validated?
+     * @param strictUserConfigValidation strict user config validation
+     */
+    public void setStrictUserConfigValidation(boolean strictUserConfigValidation) {
+        this.strictUserConfigValidation = strictUserConfigValidation;
+        this.foURIResolver.setThrowExceptions(strictUserConfigValidation);
+    }
+
+    /**
+     * Is the user configuration to be validated?
+     * @return if the user configuration should be validated
+     */
+    public boolean validateUserConfigStrictly() {
+        return this.strictUserConfigValidation;
+    }
+
+    //------------------------------------------- Cache related stuff
+
+    /**
+     * Whether or not to cache results of font triplet detection/auto-config
+     * @param useCache use cache or not
+     */
+    public void setUseCache(boolean useCache) {
+        if (useCache) {
+            this.fontCache = FontCache.load();
+            if (this.fontCache == null) {
+                this.fontCache = new FontCache();
+            }
+        } else {
+            this.fontCache = null;
+        }
+    }
+
+    /**
+     * Cache results of font triplet detection/auto-config?
+     * @return whether this factory is uses the cache
+     */
+    public boolean useCache() {
+        return (this.fontCache != null);
+    }
+
+    public FontCache getFontCache() {
+        return this.fontCache;
     }
     
-    /**
-     * Initializes user agent settings from the user configuration
-     * file, if present: baseURL, resolution, default page size,...
-     * 
-     * @throws ConfigurationException when there is an entry that 
-     *          misses the required attribute
-     */
-    public void initUserConfig() throws ConfigurationException {
-        log.debug("Initializing User Agent Configuration");
-        setFontBaseURL(getBaseURLfromConfig(userConfig, "font-base"));
-        final String hyphBase = getBaseURLfromConfig(userConfig, "hyphenation-base");
-        if (hyphBase != null) {
-            this.hyphResolver = new HyphenationTreeResolver() {
-                public Source resolve(String href) {
-                    return resolveURI(href, hyphBase);
-                }
-            };
-        }
-        if (userConfig.getChild("source-resolution", false) != null) {
-            this.sourceResolution 
-                = userConfig.getChild("source-resolution").getValueAsFloat(
-                        DEFAULT_SOURCE_RESOLUTION);
-            log.info("Source resolution set to: " + sourceResolution 
-                    + "dpi (px2mm=" + getSourcePixelUnitToMillimeter() + ")");
-        }
-        if (userConfig.getChild("strict-validation", false) != null) {
-            this.strictValidation = userConfig.getChild("strict-validation").getValueAsBoolean();
-        }
-        if (userConfig.getChild("break-indent-inheritance", false) != null) {
-            this.breakIndentInheritanceOnReferenceAreaBoundary 
-                = userConfig.getChild("break-indent-inheritance").getValueAsBoolean();
-        }
-        Configuration pageConfig = userConfig.getChild("default-page-settings");
-        if (pageConfig.getAttribute("height", null) != null) {
-            setPageHeight(pageConfig.getAttribute("height"));
-            log.info("Default page-height set to: " + pageHeight);
-        }
-        if (pageConfig.getAttribute("width", null) != null) {
-            setPageWidth(pageConfig.getAttribute("width"));
-            log.info("Default page-width set to: " + pageWidth);
-        }
-    }
-
-    /**
-     * Retrieves and verifies a base URL.
-     * @param cfg The Configuration object to retrieve the base URL from
-     * @param name the element name for the base URL
-     * @return the requested base URL or null if not available
-     */
-    public static String getBaseURLfromConfig(Configuration cfg, String name) {
-        if (cfg.getChild(name, false) != null) {
-            try {
-                String cfgBaseDir = cfg.getChild(name).getValue(null);
-                if (cfgBaseDir != null) {
-                    File dir = new File(cfgBaseDir);
-                    if (dir.isDirectory()) {
-                        cfgBaseDir = dir.toURL().toExternalForm(); 
-                    }
-                }
-                log.info(name + " set to: " + cfgBaseDir);
-                return cfgBaseDir;
-            } catch (MalformedURLException mue) {
-                log.error("Base URL in user config is malformed!");
-            }
-        }
-        return null;
-    }
-
-    //------------------------------------------- URI resolution
-
     /**
      * Attempts to resolve the given URI.
      * Will use the configured resolver and if not successful fall back
      * to the default resolver.
-     * @param uri URI to access
-     * @param base the base URI to resolve against
+     * @param href URI to access
+     * @param baseUri the base URI to resolve against
      * @return A {@link javax.xml.transform.Source} object, or null if the URI
      * cannot be resolved. 
      * @see org.apache.fop.apps.FOURIResolver
      */
-    public Source resolveURI(String uri, String base) {
+    public Source resolveURI(String href, String baseUri) {
         Source source = null;
-        //RFC 2397 data URLs don't need to be resolved, just decode them.
-        boolean bypassURIResolution = uri.startsWith("data:");
-        if (!bypassURIResolution && uriResolver != null) {
-            try {
-                source = uriResolver.resolve(uri, base);
-            } catch (TransformerException te) {
-                log.error("Attempt to resolve URI '" + uri + "' failed: ", te);
-            }
-        }
-        if (source == null) {
-            // URI Resolver not configured or returned null, use default resolver
-            try {
-                source = foURIResolver.resolve(uri, base);
-            } catch (TransformerException te) {
-                log.error("Attempt to resolve URI '" + uri + "' failed: ", te);
-            }
+        try {
+            source = foURIResolver.resolve(href, baseUri);
+        } catch (TransformerException e) {
+            log.error("Attempt to resolve URI '" + href + "' failed: ", e);
         }
         return source;
     }
@@ -652,53 +742,11 @@ public class FopFactory {
      * The FOP URI resolver is used to try and locate the ICC file. 
      * If that fails null is returned.
      * 
-     * @param base a base URI to resolve relative URIs
+     * @param baseUri a base URI to resolve relative URIs
      * @param iccProfileSrc ICC Profile source to return a ColorSpace for
      * @return ICC ColorSpace object or null if ColorSpace could not be created 
      */
-    public ColorSpace getColorSpace(String base, String iccProfileSrc) {
-        ColorSpace colorSpace = null;
-        if (!this.colorSpaceMap.containsKey(base + iccProfileSrc)) {
-            try {
-                ICC_Profile iccProfile = null;
-                // First attempt to use the FOP URI resolver to locate the ICC
-                // profile
-                Source src = this.resolveURI(iccProfileSrc, base);
-                if (src != null && src instanceof StreamSource) {
-                    // FOP URI resolver found ICC profile - create ICC profile
-                    // from the Source
-                    iccProfile = ICC_Profile.getInstance(((StreamSource) src)
-                            .getInputStream());
-                } else {
-                    // TODO - Would it make sense to fall back on VM ICC
-                    // resolution
-                    // Problem is the cache might be more difficult to maintain
-                    // 
-                    // FOP URI resolver did not find ICC profile - perhaps the
-                    // Java VM can find it?
-                    // iccProfile = ICC_Profile.getInstance(iccProfileSrc);
-                }
-                if (iccProfile != null) {
-                    colorSpace = new ICC_ColorSpace(iccProfile);
-                }
-            } catch (IOException e) {
-                // Ignore exception - will be logged a bit further down
-                // (colorSpace == null case)
-            }
-
-            if (colorSpace != null) {
-                // Put in cache (not when VM resolved it as we can't control
-                this.colorSpaceMap.put(base + iccProfileSrc, colorSpace);
-            } else {
-                // TODO To avoid an excessive amount of warnings perhaps
-                // register a null ColorMap in the colorSpaceMap
-                log.warn("Color profile '" + iccProfileSrc + "' not found.");
-            }
-        } else {
-            colorSpace = (ColorSpace) this.colorSpaceMap.get(base
-                    + iccProfileSrc);
-        }
-        return colorSpace;
-    }
-    
+    public ColorSpace getColorSpace(String baseUri, String iccProfileSrc) {
+        return colorSpaceCache.get(baseUri, iccProfileSrc);
+    }    
 }
