@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MultiProperties.java 488960 2006-12-20 08:34:28Z spepping $ */
+/* $Id: MultiProperties.java 554094 2007-07-07 00:04:25Z adelmelle $ */
 
 package org.apache.fop.fo.flow;
 
@@ -25,7 +25,6 @@ import org.xml.sax.Locator;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
-import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 import org.apache.fop.fo.properties.CommonAccessibility;
 
@@ -34,7 +33,6 @@ import org.apache.fop.fo.properties.CommonAccessibility;
  */
 public class MultiProperties extends FObj {
     // The value of properties relevant for fo:multi-properties.
-    private String id;
     // Unused but valid items, commented out for performance:
     //     private CommonAccessibility commonAccessibility;
     // End of property values
@@ -52,25 +50,11 @@ public class MultiProperties extends FObj {
         super(parent);
 
         if (!notImplementedWarningGiven) {
-            getLogger().warn("fo:multi-properties is not yet implemented.");
+            log.warn("fo:multi-properties is not yet implemented.");
             notImplementedWarningGiven = true;
         }
     }
 
-    /**
-     * @see org.apache.fop.fo.FObj#bind(PropertyList)
-     */
-    public void bind(PropertyList pList) throws FOPException {
-        id = pList.get(PR_ID).getString();
-    }
-
-    /**
-     * @see org.apache.fop.fo.FONode#startOfNode
-     */
-    protected void startOfNode() throws FOPException {
-        checkId(id);
-    }
-
     /**
      * Make sure content model satisfied, if so then tell the
      * FOEventHandler that we are at the end of the flow.
@@ -104,13 +88,6 @@ public class MultiProperties extends FObj {
                 invalidChildError(loc, nsURI, localName);
             }
     }
-
-    /**
-     * Return the "id" property.
-     */
-    public String getId() {
-        return id;
-    }
     
     /** @see org.apache.fop.fo.FONode#getLocalName() */
     public String getLocalName() {
