@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Table.java 488960 2006-12-20 08:34:28Z spepping $ */
+/* $Id: Table.java 556248 2007-07-14 08:52:28Z adelmelle $ */
 
 package org.apache.fop.fo.flow;
 
@@ -54,7 +54,6 @@ public class Table extends TableFObj {
     private LengthPairProperty borderSeparation;
     private int breakAfter;
     private int breakBefore;
-    private String id;
     private LengthRangeProperty inlineProgressionDimension;
     private KeepProperty keepTogether;
     private KeepProperty keepWithNext;
@@ -110,6 +109,7 @@ public class Table extends TableFObj {
      * @see org.apache.fop.fo.FObj#bind(PropertyList)
      */
     public void bind(PropertyList pList) throws FOPException {
+        super.bind(pList);
         commonBorderPaddingBackground = pList.getBorderPaddingBackgroundProps();
         commonMarginBlock = pList.getMarginBlockProps();
         blockProgressionDimension = pList.get(PR_BLOCK_PROGRESSION_DIMENSION).getLengthRange();
@@ -117,7 +117,6 @@ public class Table extends TableFObj {
         borderSeparation = pList.get(PR_BORDER_SEPARATION).getLengthPair();
         breakAfter = pList.get(PR_BREAK_AFTER).getEnum();
         breakBefore = pList.get(PR_BREAK_BEFORE).getEnum();
-        id = pList.get(PR_ID).getString();
         inlineProgressionDimension = pList.get(PR_INLINE_PROGRESSION_DIMENSION).getLengthRange();
         keepTogether = pList.get(PR_KEEP_TOGETHER).getKeep();
         keepWithNext = pList.get(PR_KEEP_WITH_NEXT).getKeep();
@@ -125,26 +124,22 @@ public class Table extends TableFObj {
         tableLayout = pList.get(PR_TABLE_LAYOUT).getEnum();
         tableOmitFooterAtBreak = pList.get(PR_TABLE_OMIT_FOOTER_AT_BREAK).getEnum();
         tableOmitHeaderAtBreak = pList.get(PR_TABLE_OMIT_HEADER_AT_BREAK).getEnum();
-        super.bind(pList);
 
         //Bind extension properties
         widowContentLimit = pList.get(PR_X_WIDOW_CONTENT_LIMIT).getLength();
         orphanContentLimit = pList.get(PR_X_ORPHAN_CONTENT_LIMIT).getLength();
-        
-        if (borderCollapse != EN_SEPARATE) {
-            //TODO Remove once the collapsing border is at least marginally working.
-            borderCollapse = EN_SEPARATE;
-            log.debug("A table has been forced to use the separate border model"
-                    + " (border-collapse=\"separate\") as the collapsing border model"
-                    + " is not implemented, yet.");
+
+        if (!blockProgressionDimension.getOptimum(null).isAuto()) {
+            attributeWarning("only a value of \"auto\" for block-progression-dimension has a well-specified"
+                    + " behavior on fo:table. Falling back to \"auto\"");
+            // Anyway, the bpd of a table is not used by the layout code
         }
         if (tableLayout == EN_AUTO) {
             attributeWarning("table-layout=\"auto\" is currently not supported by FOP");
         }
         if (!isSeparateBorderModel() 
                 && getCommonBorderPaddingBackground().hasPadding(
-                        ValidationPercentBaseContext
-                            .getPseudoContextForValidationPurposes())) {
+                        ValidationPercentBaseContext.getPseudoContext())) {
             //See "17.6.2 The collapsing border model" in CSS2
             attributeWarning("In collapsing border model a table does not have padding"
                     + " (see http://www.w3.org/TR/REC-CSS2/tables.html#collapsing-borders)"
@@ -163,7 +158,7 @@ public class Table extends TableFObj {
      * @see org.apache.fop.fo.FONode#startOfNode
      */
     protected void startOfNode() throws FOPException {
-        checkId(id);
+        super.startOfNode();
         getFOEventHandler().startTable(this);
     }
    
@@ -201,7 +196,7 @@ public class Table extends TableFObj {
                     tooManyNodesError(loc, "table-footer");
                 } else {
                     tableFooterFound = true;
-                    if (tableBodyFound) {
+                    if (tableBodyFound && getUserAgent().validateStrictly()) {
                         nodesOutOfOrderError(loc, "fo:table-footer", 
                             "(table-body+)");
                     }
@@ -344,19 +339,15 @@ public class Table extends TableFObj {
         return (tableLayout == EN_AUTO);
     }
 
-    /** @return the list of table-column elements. */
+    /**
+     *  Returns the list of table-column elements.
+     *
+     * @return a list of {@link TableColumn} elements, may contain null elements
+     */
     public List getColumns() {
         return columns;
     }
     
-    /**
-     * @param index index of the table-body element.
-     * @return the requested table-body element
-     */
-    public TableBody getBody(int index) {
-        return (TableBody) childNodes.get(index);
-    }
-
     /** @return the body for the table-header. */
     public TableBody getTableHeader() {
         return tableHeader;
@@ -464,11 +455,6 @@ public class Table extends TableFObj {
         return orphanContentLimit;
     }
 
-    /** @return the "id" property. */
-    public String getId() {
-        return id;
-    }
-
     /** @see org.apache.fop.fo.FONode#getLocalName() */
     public String getLocalName() {
         return "table";
