@@ -15,18 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id: Declarations.java 474225 2006-11-13 10:08:19Z jeremias $ */
+/* $Id: Declarations.java 554104 2007-07-07 01:07:10Z adelmelle $ */
 
 package org.apache.fop.fo.pagination;
 
 // Java
-import java.util.Iterator;
 import java.util.Map;
 
 import org.xml.sax.Locator;
 
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.FONode;
+import org.apache.fop.fo.FONode.FONodeIterator;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
@@ -77,23 +77,23 @@ public class Declarations extends FObj {
      * @see org.apache.fop.fo.FONode#endOfNode()
      */
     protected void endOfNode() throws FOPException {
-        if (childNodes != null) {
-            for (Iterator iter = childNodes.iterator(); iter.hasNext();) {
-                FONode node = (FONode)iter.next();
+        if (firstChild != null) {
+            for (FONodeIterator iter = getChildNodes(); iter.hasNext();) {
+                FONode node = iter.nextNode();
                 if (node.getName().equals("fo:color-profile")) {
                     ColorProfile cp = (ColorProfile)node;
                     if (!"".equals(cp.getColorProfileName())) {
                         addColorProfile(cp);
                     } else {
-                        getLogger().warn("color-profile-name required for color profile");
+                        log.warn("color-profile-name required for color profile");
                     }
                 } else {
-                    getLogger().debug("Ignoring element " + node.getName() 
+                    log.debug("Ignoring element " + node.getName() 
                             + " inside fo:declarations.");
                 }
             }
         }
-        childNodes = null;
+        firstChild = null;
     }
 
     private void addColorProfile(ColorProfile cp) {
@@ -102,7 +102,7 @@ public class Declarations extends FObj {
         }
         if (colorProfiles.get(cp.getColorProfileName()) != null) {
             // duplicate names
-            getLogger().warn("Duplicate fo:color-profile profile name: "
+            log.warn("Duplicate fo:color-profile profile name: "
                     + cp.getColorProfileName());
         }
         colorProfiles.put(cp.getColorProfileName(), cp);
