@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageSequence.java 428450 2006-08-03 16:53:41Z jeremias $ */
+/* $Id: PageSequence.java 554094 2007-07-07 00:04:25Z adelmelle $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -43,7 +43,6 @@ public class PageSequence extends FObj {
     private int letterValue;
     private char groupingSeparator;
     private int groupingSize;
-    private String id;
     private Numeric initialPageNumber;
     private int forcePageCount;
     private String masterReference;
@@ -96,13 +95,13 @@ public class PageSequence extends FObj {
      * @see org.apache.fop.fo.FObj#bind(PropertyList)
      */
     public void bind(PropertyList pList) throws FOPException {
+        super.bind(pList);
         country = pList.get(PR_COUNTRY).getString();
         format = pList.get(PR_FORMAT).getString();
         language = pList.get(PR_LANGUAGE).getString();
         letterValue = pList.get(PR_LETTER_VALUE).getEnum();
         groupingSeparator = pList.get(PR_GROUPING_SEPARATOR).getCharacter();
         groupingSize = pList.get(PR_GROUPING_SIZE).getNumber().intValue();
-        id = pList.get(PR_ID).getString();
         initialPageNumber = pList.get(PR_INITIAL_PAGE_NUMBER).getNumeric();
         forcePageCount = pList.get(PR_FORCE_PAGE_COUNT).getEnum();
         masterReference = pList.get(PR_MASTER_REFERENCE).getString();
@@ -116,6 +115,7 @@ public class PageSequence extends FObj {
      * @see org.apache.fop.fo.FONode#startOfNode()
      */
     protected void startOfNode() throws FOPException {
+        super.startOfNode();
         this.root = (Root) parent;
         flowMap = new java.util.HashMap();
 
@@ -127,15 +127,12 @@ public class PageSequence extends FObj {
                 throw new ValidationException("master-reference '" + masterReference
                    + "' for fo:page-sequence matches no"
                    + " simple-page-master or page-sequence-master", locator);
-            } else {
-                pageSequenceMaster.reset();
             }
         }
 
         this.pageNumberGenerator = new PageNumberGenerator(
                 format, groupingSeparator, groupingSize, letterValue);
 
-        checkId(id);
         getFOEventHandler().startPageSequence(this);
     }
 
@@ -396,11 +393,6 @@ public class PageSequence extends FObj {
         return (StaticContent) flowMap.get(name);
     }
 
-    /** @return the "id" property. */
-    public String getId() {
-        return id;
-    }
-
     /**
      * Accessor method for titleFO
      * @return titleFO for this object
@@ -454,8 +446,8 @@ public class PageSequence extends FObj {
             return simplePageMaster;
         }
         boolean isOddPage = ((page % 2) == 1);
-        if (getLogger().isDebugEnabled()) {
-            getLogger().debug("getNextSimplePageMaster(page=" + page
+        if (log.isDebugEnabled()) {
+            log.debug("getNextSimplePageMaster(page=" + page
                     + " isOdd=" + isOddPage 
                     + " isFirst=" + isFirstPage 
                     + " isLast=" + isLastPage 
