@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageBreakingAlgorithm.java 453310 2006-10-05 18:44:15Z spepping $ */
+/* $Id: PageBreakingAlgorithm.java 554094 2007-07-07 00:04:25Z adelmelle $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -38,7 +38,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     private static Log log = LogFactory.getLog(PageBreakingAlgorithm.class);
 
     private LayoutManager topLevelLM;
-    private PageSequenceLayoutManager.PageProvider pageProvider;
+    private PageProvider pageProvider;
     private PageBreakingLayoutListener layoutListener;
     /** List of PageBreakPosition elements. */
     private LinkedList pageBreaks = null;
@@ -94,7 +94,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     private boolean favorSinglePart = false;
     
     public PageBreakingAlgorithm(LayoutManager topLevelLM,
-                                 PageSequenceLayoutManager.PageProvider pageProvider,
+                                 PageProvider pageProvider,
                                  PageBreakingLayoutListener layoutListener,
                                  int alignment, int alignmentLast,
                                  MinOptMax footnoteSeparatorLength,
