@@ -15,11 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: TableContentLayoutManager.java 478928 2006-11-24 17:32:48Z vhennebert $ */
+/* $Id: TableContentLayoutManager.java 555651 2007-07-12 14:59:06Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.table;
 
-import java.util.Arrays;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
@@ -46,12 +45,10 @@ import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthPenalty;
 import org.apache.fop.layoutmgr.KnuthPossPosIter;
 import org.apache.fop.layoutmgr.LayoutContext;
-import org.apache.fop.layoutmgr.LayoutManager;
 import org.apache.fop.layoutmgr.ListElement;
 import org.apache.fop.layoutmgr.MinOptMaxUtil;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
-import org.apache.fop.layoutmgr.SpaceResolver;
 import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.layoutmgr.SpaceResolver.SpaceHandlingBreakPosition;
 import org.apache.fop.traits.MinOptMax;
@@ -72,7 +69,6 @@ public class TableContentLayoutManager implements PercentBaseContext {
     private LinkedList footerList;
     private int headerNetHeight = 0;
     private int footerNetHeight = 0;
-    private boolean firstBreakBeforeServed = false;
 
     private int startXOffset;
     private int usedBPD;
@@ -106,7 +102,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
     }
     
     /** @return true if the table uses the separate border model. */
-    private boolean isSeparateBorderModel() {
+    boolean isSeparateBorderModel() {
         return getTableLM().getTable().isSeparateBorderModel();
     }
     
@@ -183,12 +179,24 @@ public class TableContentLayoutManager implements PercentBaseContext {
         LinkedList returnList = getKnuthElementsForRowIterator(
                 bodyIter, context, alignment, TableRowIterator.BODY);
         if (headerAsFirst != null) {
-            returnList.add(0, headerAsFirst);
+            int insertionPoint = 0;
+            if (returnList.size() > 0 && ((ListElement)returnList.getFirst()).isForcedBreak()) {
+                insertionPoint++;
+            }
+            returnList.add(insertionPoint, headerAsFirst);
         } else if (headerAsSecondToLast != null) {
-            returnList.add(headerAsSecondToLast);
+            int insertionPoint = returnList.size();
+            if (returnList.size() > 0 && ((ListElement)returnList.getLast()).isForcedBreak()) {
+                insertionPoint--;
+            }
+            returnList.add(insertionPoint, headerAsSecondToLast);
         }
         if (footerAsLast != null) {
-            returnList.add(footerAsLast);
+            int insertionPoint = returnList.size();
+            if (returnList.size() > 0 && ((ListElement)returnList.getLast()).isForcedBreak()) {
+                insertionPoint--;
+            }
+            returnList.add(insertionPoint, footerAsLast);
         }
         return returnList;
     }
@@ -217,27 +225,27 @@ public class TableContentLayoutManager implements PercentBaseContext {
                         KnuthPenalty pen = (KnuthPenalty)last;
                         pen.setP(-KnuthPenalty.INFINITE);
                         pen.setBreakClass(rowFO.getBreakBefore());
-                    } else if (last instanceof BreakElement) {
+                    } else {//if (last instanceof BreakElement) { // TODO vh: seems the only possibility
                         BreakElement breakPoss = (BreakElement) last;
                         breakPoss.setPenaltyValue(-KnuthPenalty.INFINITE);
                         breakPoss.setBreakClass(rowFO.getBreakBefore());
                     }
                 } else {
-                    if (!firstBreakBeforeServed) {
-                        returnList.add(new BreakElement(new Position(getTableLM()),
-                                0, -KnuthPenalty.INFINITE, rowFO.getBreakBefore(), context));
-                        iter.backToPreviousRow();
-                        firstBreakBeforeServed = true;
-                        break;
-                    }
+                    returnList.add(new BreakElement(new Position(getTableLM()),
+                            0, -KnuthPenalty.INFINITE, rowFO.getBreakBefore(), context));
                 }
             }
-            firstBreakBeforeServed = true;
             
             //Border resolution
             if (!isSeparateBorderModel()) {
                 resolveNormalBeforeAfterBordersForRowGroup(rowGroup, iter);
             }
+
+            //Reset keep-with-next when remaining inside the table.
+            //The context flag is only used to propagate keep-with-next to the outside.
+            //The clearing is ok here because createElementsForRowGroup already handles
+            //the keep when inside a table.
+            context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING, false);
             
             //Element list creation
             createElementsForRowGroup(context, alignment, bodyType, 
@@ -281,7 +289,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
         }
         
         if (returnList.size() > 0) {
-            //Remove last penalty
+            //Remove the last penalty produced by the combining algorithm (see TableStepper), for the last step
             ListElement last = (ListElement)returnList.getLast();
             if (last.isPenalty() || last instanceof BreakElement) {
                 if (!last.isForcedBreak()) {
@@ -329,7 +337,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
                 //whole body iterator to be prefetched!
                 prevRow = this.bodyIter.getLastRow();
             }
-            log.debug(prevRow + " - " + row + " - " + nextRow);
+            log.debug("prevRow-row-nextRow: " + prevRow + " - " + row + " - " + nextRow);
             
             //Determine the grid units necessary for getting all the borders right
             int guCount = row.getGridUnits().size();
@@ -403,7 +411,6 @@ public class TableContentLayoutManager implements PercentBaseContext {
                                 CommonBorderPaddingBackground.AFTER, flags);
                     }
                 }
-
             }
         }
     }
@@ -432,10 +439,13 @@ public class TableContentLayoutManager implements PercentBaseContext {
             
             pgus.clear();
             TableRow tableRow = null;
+            // The row's minimum content height; 0 if the row's height is auto, otherwise
+            // the .minimum component of the explicitely specified value
             int minContentHeight = 0;
             int maxCellHeight = 0;
             int effRowContentHeight = 0;
             for (int j = 0; j < row.getGridUnits().size(); j++) {
+//                assert maxColumnCount == 0 || maxColumnCount == row.getGridUnits().size(); // TODO vh
                 maxColumnCount = Math.max(maxColumnCount, row.getGridUnits().size());
                 GridUnit gu = row.getGridUnit(j);
                 if ((gu.isPrimary() || (gu.getColSpanIndex() == 0 && gu.isLastGridUnitRowSpan())) 
@@ -610,12 +620,11 @@ public class TableContentLayoutManager implements PercentBaseContext {
      */
     public void addAreas(PositionIterator parentIter, LayoutContext layoutContext) {
         this.usedBPD = 0;
-        RowPainter painter = new RowPainter(layoutContext);
+        RowPainter painter = new RowPainter(this, layoutContext);
 
         List positions = new java.util.ArrayList();
         List headerElements = null;
         List footerElements = null;
-        int nestedPenaltyArea = 0;
         Position firstPos = null;
         Position lastPos = null;
         Position lastCheckPos = null;
@@ -665,7 +674,6 @@ public class TableContentLayoutManager implements PercentBaseContext {
             if (penaltyPos.footerElements != null) {
                 footerElements = penaltyPos.footerElements; 
             }
-            nestedPenaltyArea = penaltyPos.nestedPenaltyLength;
         }
 
         Map markers = getTableLM().getTable().getMarkers();
@@ -679,23 +687,18 @@ public class TableContentLayoutManager implements PercentBaseContext {
             //be handled first before all other TableContentPositions
             PositionIterator nestedIter = new KnuthPossPosIter(headerElements);
             iterateAndPaintPositions(nestedIter, painter);
-            painter.addAreasAndFlushRow(true);
         }
         
         //Iterate over all steps
         Iterator posIter = positions.iterator();
         iterateAndPaintPositions(posIter, painter);
-        painter.addAreasAndFlushRow(true);
 
-        painter.notifyNestedPenaltyArea(nestedPenaltyArea);
         if (footerElements != null) {
             //Positions for footers are simply added at the end
             PositionIterator nestedIter = new KnuthPossPosIter(footerElements);
             iterateAndPaintPositions(nestedIter, painter);
-            painter.addAreasAndFlushRow(true);
         }
         
-        painter.notifyEndOfSequence();
         this.usedBPD += painter.getAccumulatedBPD();
 
         if (markers != null) {
@@ -703,11 +706,18 @@ public class TableContentLayoutManager implements PercentBaseContext {
                     false, getTableLM().isFirst(firstPos), getTableLM().isLast(lastCheckPos));
         }
     }
-    
+
+    /**
+     * Iterates over a part of the table (header, footer, body) and paints the related
+     * elements.
+     * 
+     * @param iterator iterator over Position elements. Those positions correspond to the
+     * elements of the table present on the current page
+     * @param painter
+     */
     private void iterateAndPaintPositions(Iterator iterator, RowPainter painter) {
         List lst = new java.util.ArrayList();
         boolean firstPos = false;
-        boolean lastPos = false;
         TableBody body = null;
         while (iterator.hasNext()) {
             Position pos = (Position)iterator.next();
@@ -718,25 +728,17 @@ public class TableContentLayoutManager implements PercentBaseContext {
                 if (body == null) {
                     body = part.pgu.getBody();
                 }
-                if (tcpos.getFlag(TableContentPosition.FIRST_IN_ROWGROUP) 
+                if (tcpos.getFlag(TableContentPosition.FIRST_IN_ROWGROUP)
                         && tcpos.row.getFlag(EffRow.FIRST_IN_PART)) {
                     firstPos = true;
 
                 }
                 if (tcpos.getFlag(TableContentPosition.LAST_IN_ROWGROUP) 
                         && tcpos.row.getFlag(EffRow.LAST_IN_PART)) {
-                    lastPos = true;
-                    getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
-                            true, firstPos, lastPos);
-                    int size = lst.size();
-                    for (int i = 0; i < size; i++) {
-                        painter.handleTableContentPosition((TableContentPosition)lst.get(i));
-                    }
-                    getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
-                            false, firstPos, lastPos);
+                    log.trace("LAST_IN_ROWGROUP + LAST_IN_PART");
+                    handleMarkersAndPositions(lst, body, firstPos, true, painter);
                     //reset
                     firstPos = false;
-                    lastPos = false;
                     body = null;
                     lst.clear();
                 }
@@ -747,251 +749,24 @@ public class TableContentLayoutManager implements PercentBaseContext {
             }
         }
         if (body != null) {
-            getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
-                    true, firstPos, lastPos);
-            int size = lst.size();
-            for (int i = 0; i < size; i++) {
-                painter.handleTableContentPosition((TableContentPosition)lst.get(i));
-            }
-            getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
-                    false, firstPos, lastPos);
+            // Entering this block means that the end of the current table-part hasn't
+            // been reached (otherwise it would have been caught by the test above). So
+            // lastPos is necessarily false
+            handleMarkersAndPositions(lst, body, firstPos, false, painter);
         }
+        painter.addAreasAndFlushRow(true);
     }
-   
-    private class RowPainter {
-        
-        private TableRow rowFO = null;
-        private int colCount = getColumns().getColumnCount();
-        private int yoffset = 0;
-        private int accumulatedBPD = 0;
-        private EffRow lastRow = null;
-        private LayoutContext layoutContext;
-        private int lastRowHeight = 0;
-        private int[] firstRow = new int[3];
-        private Map[] rowOffsets = new Map[] {new java.util.HashMap(), 
-                new java.util.HashMap(), new java.util.HashMap()};
 
-        //These three variables are our buffer to recombine the individual steps into cells
-        private PrimaryGridUnit[] gridUnits = new PrimaryGridUnit[colCount];
-        private int[] start = new int[colCount];
-        private int[] end = new int[colCount];
-        private int[] partLength = new int[colCount];
-        
-        public RowPainter(LayoutContext layoutContext) {
-            this.layoutContext = layoutContext;
-            Arrays.fill(firstRow, -1);
-            Arrays.fill(end, -1);
+    private void handleMarkersAndPositions(List positions, TableBody body, boolean firstPos,
+            boolean lastPos, RowPainter painter) {
+        getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
+                true, firstPos, lastPos);
+        int size = positions.size();
+        for (int i = 0; i < size; i++) {
+            painter.handleTableContentPosition((TableContentPosition)positions.get(i));
         }
-        
-        public int getAccumulatedBPD() {
-            return this.accumulatedBPD;
-        }
-        
-        public void notifyEndOfSequence() {
-            this.accumulatedBPD += lastRowHeight; //for last row
-        }
-        
-        public void notifyNestedPenaltyArea(int length) {
-            this.lastRowHeight += length;
-        }
-        
-        public void handleTableContentPosition(TableContentPosition tcpos) {
-            if (lastRow != tcpos.row && lastRow != null) {
-                addAreasAndFlushRow(false);
-                yoffset += lastRowHeight;
-                this.accumulatedBPD += lastRowHeight;
-            }
-            if (log.isDebugEnabled()) {
-                log.debug("===handleTableContentPosition(" + tcpos);
-            }
-            rowFO = tcpos.row.getTableRow();
-            lastRow = tcpos.row;
-            Iterator partIter = tcpos.gridUnitParts.iterator();
-            //Iterate over all grid units in the current step
-            while (partIter.hasNext()) {
-                GridUnitPart gup = (GridUnitPart)partIter.next();
-                if (log.isDebugEnabled()) {
-                    log.debug(">" + gup);
-                }
-                int colIndex = gup.pgu.getStartCol();
-                if (gridUnits[colIndex] != gup.pgu) {
-                    if (gridUnits[colIndex] != null) {
-                        log.warn("Replacing GU in slot " + colIndex 
-                                + ". Some content may not be painted.");
-                    }
-                    gridUnits[colIndex] = gup.pgu;
-                    start[colIndex] = gup.start;
-                    end[colIndex] = gup.end;
-                } else {
-                    if (gup.end < end[colIndex]) {
-                        throw new IllegalStateException("Internal Error: stepper problem");
-                    }
-                    end[colIndex] = gup.end;
-                }
-            }
-        }
-        
-        public int addAreasAndFlushRow(boolean forcedFlush) {
-            int actualRowHeight = 0;
-            int readyCount = 0;
-            
-            int bt = lastRow.getBodyType();
-            if (log.isDebugEnabled()) {
-                log.debug("Remembering yoffset for row " + lastRow.getIndex() + ": " + yoffset);
-            }
-            rowOffsets[bt].put(new Integer(lastRow.getIndex()), new Integer(yoffset));
-
-            for (int i = 0; i < gridUnits.length; i++) {
-                if ((gridUnits[i] != null) 
-                        && (forcedFlush || (end[i] == gridUnits[i].getElements().size() - 1))) {
-                    if (log.isTraceEnabled()) {
-                        log.trace("getting len for " + i + " " 
-                                + start[i] + "-" + end[i]);
-                    }
-                    readyCount++;
-                    int len = ElementListUtils.calcContentLength(
-                            gridUnits[i].getElements(), start[i], end[i]);
-                    partLength[i] = len;
-                    if (log.isTraceEnabled()) {
-                        log.trace("len of part: " + len);
-                    }
-
-                    if (start[i] == 0) {
-                        LengthRangeProperty bpd = gridUnits[i].getCell()
-                                .getBlockProgressionDimension();
-                        if (!bpd.getMinimum(getTableLM()).isAuto()) {
-                            int min = bpd.getMinimum(getTableLM())
-                                        .getLength().getValue(getTableLM()); 
-                            if (min > 0) {
-                                len = Math.max(len, bpd.getMinimum(getTableLM())
-                                        .getLength().getValue(getTableLM()));
-                            }
-                        }
-                        if (!bpd.getOptimum(getTableLM()).isAuto()) {
-                            int opt = bpd.getOptimum(getTableLM())
-                                        .getLength().getValue(getTableLM());
-                            if (opt > 0) {
-                                len = Math.max(len, opt);
-                            }
-                        }
-                        if (gridUnits[i].getRow() != null) {
-                            bpd = gridUnits[i].getRow().getBlockProgressionDimension();
-                            if (!bpd.getMinimum(getTableLM()).isAuto()) {
-                                int min = bpd.getMinimum(getTableLM()).getLength()
-                                            .getValue(getTableLM()); 
-                                if (min > 0) {
-                                    len = Math.max(len, min);
-                                }
-                            }
-                        }
-                    }
-                    
-                    // Add the padding if any
-                    len += gridUnits[i].getBorders()
-                                    .getPaddingBefore(false, gridUnits[i].getCellLM());
-                    len += gridUnits[i].getBorders()
-                                    .getPaddingAfter(false, gridUnits[i].getCellLM());
-
-                    //Now add the borders to the contentLength
-                    if (isSeparateBorderModel()) {
-                        len += gridUnits[i].getBorders().getBorderBeforeWidth(false);
-                        len += gridUnits[i].getBorders().getBorderAfterWidth(false);
-                    }
-                    int startRow = Math.max(gridUnits[i].getStartRow(), firstRow[bt]);
-                    Integer storedOffset = (Integer)rowOffsets[bt].get(new Integer(startRow));
-                    int effYOffset;
-                    if (storedOffset != null) {
-                        effYOffset = storedOffset.intValue();
-                    } else {
-                        effYOffset = yoffset;
-                    }
-                    len -= yoffset - effYOffset;
-                    actualRowHeight = Math.max(actualRowHeight, len);
-                }
-            }
-            if (readyCount == 0) {
-                return 0;
-            }
-            actualRowHeight += 2 * getTableLM().getHalfBorderSeparationBPD();
-            lastRowHeight = actualRowHeight;
-            
-            //Add areas for row
-            addRowBackgroundArea(rowFO, actualRowHeight, layoutContext.getRefIPD(), yoffset);
-            for (int i = 0; i < gridUnits.length; i++) {
-                GridUnit currentGU = lastRow.safelyGetGridUnit(i);
-                if ((gridUnits[i] != null) 
-                        && (forcedFlush || ((end[i] == gridUnits[i].getElements().size() - 1))
-                                && (currentGU == null || currentGU.isLastGridUnitRowSpan()))
-                    || (gridUnits[i] == null && currentGU != null)) {
-                    //the last line in the "if" above is to avoid a premature end of an 
-                    //row-spanned cell because no GridUnitParts are generated after a cell is
-                    //finished with its content. currentGU can be null if there's no grid unit
-                    //at this place in the current row (empty cell and no borders to process)
-                    if (log.isDebugEnabled()) {
-                        log.debug((forcedFlush ? "FORCED " : "") + "flushing..." + i + " " 
-                                + start[i] + "-" + end[i]);
-                    }
-                    PrimaryGridUnit gu = gridUnits[i];
-                    if (gu == null 
-                            && !currentGU.isEmpty() 
-                            && currentGU.getColSpanIndex() == 0 
-                            && currentGU.isLastGridUnitColSpan()
-                            && (forcedFlush || currentGU.isLastGridUnitRowSpan())) {
-                        gu = currentGU.getPrimary();
-                    }
-                    if (gu != null) {
-                        addAreasForCell(gu, start[i], end[i], 
-                                lastRow,  
-                                partLength[i], actualRowHeight);
-                        gridUnits[i] = null;
-                        start[i] = 0;
-                        end[i] = -1;
-                        partLength[i] = 0;
-                    }
-                }
-            }
-            return actualRowHeight;
-        }
-
-        private void addAreasForCell(PrimaryGridUnit pgu, int startPos, int endPos, 
-                EffRow row, int contentHeight, int rowHeight) {
-            int bt = row.getBodyType();
-            if (firstRow[bt] < 0) {
-                firstRow[bt] = row.getIndex();
-            }
-            //Determine the first row in this sequence
-            int startRow = Math.max(pgu.getStartRow(), firstRow[bt]);
-            //Determine y offset for the cell
-            Integer offset = (Integer)rowOffsets[bt].get(new Integer(startRow));
-            while (offset == null) {
-                startRow--;
-                offset = (Integer)rowOffsets[bt].get(new Integer(startRow));
-            }
-            int effYOffset = offset.intValue();
-            int effCellHeight = rowHeight;
-            effCellHeight += yoffset - effYOffset;
-            if (log.isDebugEnabled()) {
-                log.debug("Creating area for cell:");
-                log.debug("  current row: " + row.getIndex());
-                log.debug("  start row: " + pgu.getStartRow() + " " + yoffset + " " + effYOffset);
-                log.debug("  contentHeight: " + contentHeight + " rowHeight=" + rowHeight 
-                        + " effCellHeight=" + effCellHeight);
-            }
-            TableCellLayoutManager cellLM = pgu.getCellLM();
-            cellLM.setXOffset(getXOffsetOfGridUnit(pgu));
-            cellLM.setYOffset(effYOffset);
-            cellLM.setContentHeight(contentHeight);
-            cellLM.setRowHeight(effCellHeight);
-            //cellLM.setRowHeight(row.getHeight().opt);
-            int prevBreak = ElementListUtils.determinePreviousBreak(pgu.getElements(), startPos);
-            if (endPos >= 0) {
-                SpaceResolver.performConditionalsNotification(pgu.getElements(), 
-                        startPos, endPos, prevBreak);
-            }
-            cellLM.addAreas(new KnuthPossPosIter(pgu.getElements(), 
-                    startPos, endPos + 1), layoutContext);
-        }
-        
+        getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
+                false, firstPos, lastPos);
     }
 
     /**
@@ -1046,199 +821,9 @@ public class TableContentLayoutManager implements PercentBaseContext {
     public int getUsedBPD() {
         return this.usedBPD;
     }
-    
-    /**
-     * Represents a non-dividable part of a grid unit. Used by the table stepper.
-     */
-    protected static class GridUnitPart {
-        
-        /** Primary grid unit */
-        protected PrimaryGridUnit pgu;
-        /** Index of the starting element of this part */
-        protected int start;
-        /** Index of the ending element of this part */
-        protected int end;
-        
-        /**
-         * Creates a new GridUnitPart.
-         * @param pgu Primary grid unit
-         * @param start starting element
-         * @param end ending element
-         */
-        protected GridUnitPart(PrimaryGridUnit pgu, int start, int end) {
-            this.pgu = pgu;
-            this.start = start;
-            this.end = end;
-        }
-        
-        /** @return true if this part is the first part of a cell */
-        public boolean isFirstPart() {
-            return (start == 0);
-        }
-        
-        /** @return true if this part is the last part of a cell */
-        public boolean isLastPart() {
-            return (end >= 0 && end == pgu.getElements().size() - 1);
-        }
-        
-        /** @see java.lang.Object#toString() */
-        public String toString() {
-            StringBuffer sb = new StringBuffer("Part: ");
-            sb.append(start).append("-").append(end);
-            sb.append(" [").append(isFirstPart() ? "F" : "-").append(isLastPart() ? "L" : "-");
-            sb.append("] ").append(pgu);
-            return sb.toString();
-        }
-        
-    }
-    
-    /**
-     * This class represents a Position specific to this layout manager. Used for normal content
-     * cases.
-     */
-    public static class TableContentPosition extends Position {
 
-        /** The position is the first of the row group. */ 
-        public static final int FIRST_IN_ROWGROUP = 1;
-        /** The position is the last of the row group. */ 
-        public static final int LAST_IN_ROWGROUP = 2;
-        
-        /** the list of GridUnitParts making up this position */
-        protected List gridUnitParts;
-        /** effective row this position belongs to */
-        protected EffRow row;
-        /** flags for the position */
-        protected int flags;
-        
-        /**
-         * Creates a new TableContentPosition.
-         * @param lm applicable layout manager
-         * @param gridUnitParts the list of GridUnitPart instances
-         * @param row effective row this position belongs to
-         */
-        protected TableContentPosition(LayoutManager lm, List gridUnitParts, 
-                EffRow row) {
-            super(lm);
-            this.gridUnitParts = gridUnitParts;
-            this.row = row;
-        }
-        
-        /**
-         * Returns a flag for this GridUnit.
-         * @param which the requested flag
-         * @return the value of the flag
-         */
-        public boolean getFlag(int which) {
-            return (flags & (1 << which)) != 0;
-        }
-        
-        /**
-         * Sets a flag on a GridUnit.
-         * @param which the flag to set
-         * @param value the new value for the flag
-         */
-        public void setFlag(int which, boolean value) {
-            if (value) {
-                flags |= (1 << which); //set flag
-            } else {
-                flags &= ~(1 << which); //clear flag
-            }
-        }
-        
-        /** @see org.apache.fop.layoutmgr.Position#generatesAreas() */
-        public boolean generatesAreas() {
-            return true;
-        }
-
-        /** @see java.lang.Object#toString() */
-        public String toString() {
-            StringBuffer sb = new StringBuffer("TableContentPosition:");
-            sb.append(getIndex());
-            sb.append("[");
-            sb.append(row.getIndex()).append("/");
-            sb.append(getFlag(FIRST_IN_ROWGROUP) ? "F" : "-");
-            sb.append(getFlag(LAST_IN_ROWGROUP) ? "L" : "-").append("]");
-            sb.append("(");
-            sb.append(gridUnitParts);
-            sb.append(")");
-            return sb.toString();
-        }
-    }
-    
-    /**
-     * This class represents a Position specific to this layout manager. Used for table
-     * headers and footers at the beginning and end of a table.
-     */
-    public static class TableHeaderFooterPosition extends Position {
-
-        /** True indicates a position for a header, false for a footer. */
-        protected boolean header;
-        /** Element list representing the header/footer */
-        protected List nestedElements;
-        
-        /**
-         * Creates a new TableHeaderFooterPosition.
-         * @param lm applicable layout manager
-         * @param header True indicates a position for a header, false for a footer.
-         * @param nestedElements Element list representing the header/footer
-         */
-        protected TableHeaderFooterPosition(LayoutManager lm, 
-                boolean header, List nestedElements) {
-            super(lm);
-            this.header = header;
-            this.nestedElements = nestedElements;
-        }
-        
-        /** @see java.lang.Object#toString() */
-        public String toString() {
-            StringBuffer sb = new StringBuffer("Table");
-            sb.append(header ? "Header" : "Footer");
-            sb.append("Position:");
-            sb.append(getIndex()).append("(");
-            sb.append(nestedElements);
-            sb.append(")");
-            return sb.toString();
-        }
-    }
-
-    /**
-    * This class represents a Position specific to this layout manager. Used for table
-    * headers and footers at breaks.
-    */
-    public static class TableHFPenaltyPosition extends Position {
-        
-        /** Element list for the header */
-        protected List headerElements;
-        /** Element list for the footer */
-        protected List footerElements;
-        /** Penalty length to be respected for nested content */
-        protected int nestedPenaltyLength;
-        
-        /**
-         * Creates a new TableHFPenaltyPosition
-         * @param lm applicable layout manager
-         */
-        protected TableHFPenaltyPosition(LayoutManager lm) {
-            super(lm);
-        }
-        
-        /** @see java.lang.Object#toString() */
-        public String toString() {
-            StringBuffer sb = new StringBuffer("TableHFPenaltyPosition:");
-            sb.append(getIndex()).append("(");
-            sb.append("header:");
-            sb.append(headerElements);
-            sb.append(", footer:");
-            sb.append(footerElements);
-            sb.append(", inner penalty length:");
-            sb.append(nestedPenaltyLength);
-            sb.append(")");
-            return sb.toString();
-        }
-    }
-    
     // --------- Property Resolution related functions --------- //
-    
+
     /**
      * @see org.apache.fop.datatypes.PercentBaseContext#getBaseLength(int, FObj)
      */
