@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableRowIterator.java 478928 2006-11-24 17:32:48Z vhennebert $ */
+/* $Id: TableRowIterator.java 532780 2007-04-26 15:32:37Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -35,7 +35,7 @@ import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 
 
 /**
- * Iterator that lets the table layout manager step over all of the rows of a part of the
+ * Iterator that lets the table layout manager step over all the rows of a part of the
  * table (table-header, table-footer or table-body).
  * <p>Note: This class is not thread-safe.</p>
  */
@@ -72,12 +72,14 @@ public class TableRowIterator {
     /** Index of the row currently being fetched. */
     private int fetchIndex = -1;
 
-    /** Spans found on the current row which will also span over the next row. */
+    /**
+     * Number of spans found on the current row which will also span over the next row.
+     */
     private int pendingRowSpans;
 
     //TODO rows should later be a Jakarta Commons LinkedList so concurrent modifications while
     //using a ListIterator are possible
-    /** List of cached rows. This a list of EffRow elements. */
+    /** List of cached rows. This is a list of EffRow elements. */
     private List fetchedRows = new java.util.ArrayList();
 
     /**
@@ -125,16 +127,6 @@ public class TableRowIterator {
         }
     }
 
-    /**
-     * Preloads the whole table.
-     * <p>Note:This is inefficient for large tables.</p>
-     */
-    public void prefetchAll() {
-        while (prefetchNext()) {
-            log.trace("found row...");
-        }
-    }
-
     /**
      * Returns the next row group if any. A row group in this context is the minimum number of
      * consecutive rows which contains all spanned grid units of its cells.
@@ -219,13 +211,6 @@ public class TableRowIterator {
         return getRow(row.getIndex() + 1);
     }
 
-    /**
-     * Sets the iterator to the previous row.
-     */
-    public void backToPreviousRow() {
-        iteratorIndex--;
-    }
-
     /**
      * Returns the first effective row.
      * @return the requested effective row.
@@ -259,7 +244,7 @@ public class TableRowIterator {
      * @return the requested effective row or null if (index &lt; 0 || index &gt;= the
      * number of already fetched rows)
      */
-    public EffRow getCachedRow(int index) {
+    private EffRow getCachedRow(int index) {
         if (index < 0 || index >= fetchedRows.size()) {
             return null;
         } else {
@@ -387,7 +372,7 @@ public class TableRowIterator {
     /**
      * Builds the list of grid units corresponding to the given table row.
      *
-     * @param cells list of cells belonging to the row
+     * @param cells list of cells starting at the current row
      * @param rowFO the fo:table-row object containing the row, possibly null
      * @return the list of grid units
      */
@@ -485,7 +470,7 @@ public class TableRowIterator {
                     safelySetListItem(gridUnits, colnum - 1, guSpan);
                     if (hasRowSpanningLeft) {
                         pendingRowSpans++;
-                        safelySetListItem(previousRowsSpanningCells, colnum - 1, gu);
+                        safelySetListItem(previousRowsSpanningCells, colnum - 1, guSpan);
                     }
                     horzSpan[j] = guSpan;
                 }
