@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFDocument.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: PDFDocument.java 535883 2007-05-07 14:30:23Z jeremias $ */
 
 package org.apache.fop.pdf;
 
@@ -28,6 +28,7 @@ import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
+import java.util.Collections;
 import java.util.Date;
 import java.util.List;
 import java.util.Map;
@@ -201,6 +202,11 @@ public class PDFDocument {
      */
     protected List links = new java.util.ArrayList();
 
+    /**
+     * List of Destinations.
+     */
+    protected List destinations;
+
     /**
      * List of FileSpecs.
      */
@@ -216,6 +222,12 @@ public class PDFDocument {
      */
     protected List gotos = new java.util.ArrayList();
 
+    /**
+     * The PDFDests object for the name dictionary.
+     * Note: This object is not a list.
+     */
+    private PDFDests dests;
+
     private PDFFactory factory;
 
     private boolean encodingOnTheFly = true;
@@ -577,6 +589,20 @@ public class PDFDocument {
         return (PDFFont)fontMap.get(fontname);
     }
 
+    /**
+     * Finds a named destination.
+     * @param compare reference object to use as search template
+     * @return the link if found, null otherwise
+     */
+    protected PDFDestination findDestination(PDFDestination compare) {
+        int index = getDestinationList().indexOf(compare);
+        if (index >= 0) {
+            return (PDFDestination)getDestinationList().get(index);
+        } else {
+            return null;
+        }
+    }
+
     /**
      * Finds a link.
      * @param compare reference object to use as search template
@@ -701,6 +727,48 @@ public class PDFDocument {
         return xObject;
     }
 
+    /**
+     * Gets the PDFDests object (which represents the /Dests entry).
+     *
+     * @return the PDFDests object (which represents the /Dests entry).
+     */
+    public PDFDests getDests() {
+        return dests;
+    }
+
+    /**
+     * Adds a destination to the document.
+     * @param destination the destination object
+     */
+    public void addDestination(PDFDestination destination) {
+        if (this.destinations == null) {
+            this.destinations = new java.util.ArrayList();
+        }
+        this.destinations.add(destination);
+    }
+    
+    /**
+     * Gets the list of named destinations.
+     *
+     * @return the list of named destinations.
+     */
+    public List getDestinationList() {
+        if (hasDestinations()) {
+            return destinations;
+        } else {
+            return Collections.EMPTY_LIST;
+        }
+    }
+
+    /**
+     * Gets whether the document has named destinations.
+     *
+     * @return whether the document has named destinations.
+     */
+    public boolean hasDestinations() {
+        return this.destinations != null && !this.destinations.isEmpty();
+    }
+
     /**
      * Add an image to the PDF document.
      * This adds an image to the PDF objects.
@@ -849,7 +917,7 @@ public class PDFDocument {
 
         getProfile().verifyPDFVersion();
         
-        byte[] pdf = ("%PDF-" + getPDFVersionString() + "\n").getBytes();
+        byte[] pdf = encode("%PDF-" + getPDFVersionString() + "\n");
         stream.write(pdf);
         this.position += pdf.length;
 
@@ -870,9 +938,9 @@ public class PDFDocument {
         try {
             MessageDigest digest = MessageDigest.getInstance("MD5");
             DateFormat df = new SimpleDateFormat("yyyy'-'MM'-'dd'T'HH':'mm':'ss'.'SSS");
-            digest.update(df.format(new Date()).getBytes());
+            digest.update(encode(df.format(new Date())));
             //Ignoring the filename here for simplicity even though it's recommended by the PDF spec
-            digest.update(String.valueOf(this.position).getBytes());
+            digest.update(encode(String.valueOf(this.position)));
             digest.update(getInfo().toPDF());
             byte[] res = digest.digest();
             String s = PDFText.toHex(res);
@@ -893,6 +961,14 @@ public class PDFDocument {
      * @throws IOException if there is an exception writing to the output stream
      */
     public void outputTrailer(OutputStream stream) throws IOException {
+        if (hasDestinations()) {
+            Collections.sort(destinations, new DestinationComparator());
+            dests = getFactory().makeDests(destinations);
+            if (this.root.getNames() == null) {
+                this.root.setNames(getFactory().makeNames());
+            }
+            this.root.getNames().setDests(dests);
+        }
         output(stream);
         for (int count = 0; count < trailerObjects.size(); count++) {
             PDFObject o = (PDFObject)trailerObjects.get(count);
