@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFactory.java 454725 2006-10-10 13:00:05Z bdelacretaz $ */
+/* $Id: PDFFactory.java 535883 2007-05-07 14:30:23Z jeremias $ */
 
 package org.apache.fop.pdf;
 
 // Java
+import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.io.FileNotFoundException;
 import java.io.IOException;
@@ -31,6 +32,7 @@ import java.util.List;
 import java.util.Map;
 import javax.xml.transform.Source;
 import javax.xml.transform.stream.StreamSource;
+import java.util.ArrayList;
 
 // Apache libs
 import org.apache.commons.io.IOUtils;
@@ -141,7 +143,7 @@ public class PDFFactory {
 
     /**
      * Make a Metadata object.
-     * @param doc the DOM Document containing the XMP metadata.
+     * @param meta the DOM Document containing the XMP metadata.
      * @param readOnly true if the metadata packet should be marked read-only
      * @return the newly created Metadata object
      */
@@ -731,7 +733,7 @@ public class PDFFactory {
         PDFPattern myPattern;
         //PDFColorSpace theColorSpace;
         double interpolation = (double)1.000;
-        List theFunctions = new java.util.ArrayList();
+        List theFunctions = new ArrayList();
 
         int currentPosition;
         int lastPosition = theColors.size() - 1;
@@ -743,13 +745,11 @@ public class PDFFactory {
 
         for (currentPosition = 0; currentPosition < lastPosition;
                 currentPosition++) {    // for every consecutive color pair
-            PDFColor currentColor =
-                (PDFColor)theColors.get(currentPosition);
+            PDFColor currentColor = (PDFColor)theColors.get(currentPosition);
             PDFColor nextColor = (PDFColor)theColors.get(currentPosition
                                  + 1);
             // colorspace must be consistant
-            if (getDocument().getColorSpace()
-                    != currentColor.getColorSpace()) {
+            if (getDocument().getColorSpace() != currentColor.getColorSpace()) {
                 currentColor.setColorSpace(
                     getDocument().getColorSpace());
             }
@@ -782,7 +782,7 @@ public class PDFFactory {
             } else {    // if the center x, center y, and radius specifiy
                 // the gradient, then assume the same center x, center y,
                 // and radius of zero for the other necessary component
-                List newCoords = new java.util.ArrayList();
+                List newCoords = new ArrayList();
                 newCoords.add(theCoords.get(0));
                 newCoords.add(theCoords.get(1));
                 newCoords.add(theCoords.get(2));
@@ -809,7 +809,118 @@ public class PDFFactory {
         return (myPattern);
     }
 
+    /* ============= named destinations and the name dictionary ============ */
+
+    /**
+     * Registers and returns newdest if it is unique. Otherwise, returns
+     * the equal destination already present in the document.
+     *
+     * @param newdest a new, as yet unregistered destination
+     * @return newdest if unique, else the already registered instance
+     */
+    protected PDFDestination getUniqueDestination(PDFDestination newdest) {
+        PDFDestination existing = getDocument().findDestination(newdest);
+        if (existing != null) {
+            return existing;
+        } else {
+            getDocument().addDestination(newdest);
+            return newdest;
+        }
+    }
+
+    /**
+     * Make a named destination.
+     *
+     * @param idRef ID Reference for this destination (the name of the destination)
+     * @param goToRef Object reference to the GoTo Action
+     * @return the newly created destrination
+     */
+    public PDFDestination makeDestination(String idRef, Object goToRef) {
+        PDFDestination destination = new PDFDestination(idRef, goToRef);
+        return getUniqueDestination(destination);
+    }
+
+    /**
+     * Make a names dictionary (the /Names object).
+     * @return the new PDFNames object
+     */
+    public PDFNames makeNames() {
+        PDFNames names = new PDFNames();
+        getDocument().registerObject(names);
+        return names;
+    }
+
+    /**
+     * Make a the head object of the name dictionary (the /Dests object).
+     *
+     * @param destinationList a list of PDFDestination instances
+     * @return the new PDFDests object
+     */
+    public PDFDests makeDests(List destinationList) {
+        PDFDests dests;
+        
+        final boolean deep = true;
+        //true for a "deep" structure (one node per entry), true for a "flat" structure
+        if (deep) {
+            dests = new PDFDests();
+            PDFArray kids = new PDFArray();
+            Iterator iter = destinationList.iterator();
+            while (iter.hasNext()) {
+                PDFDestination dest = (PDFDestination)iter.next();
+                PDFNameTreeNode node = new PDFNameTreeNode();
+                getDocument().registerObject(node);
+                node.setLowerLimit(dest.getIDRef());
+                node.setUpperLimit(dest.getIDRef());
+                node.setNames(new PDFArray());
+                node.getNames().add(dest);
+                kids.add(node);
+            }
+            dests.setLowerLimit(((PDFNameTreeNode)kids.get(0)).getLowerLimit());
+            dests.setUpperLimit(((PDFNameTreeNode)kids.get(kids.length() - 1)).getUpperLimit());
+            dests.setKids(kids);
+        } else {
+            dests = new PDFDests(destinationList);
+        }
+        getDocument().registerObject(dests);
+        return dests;
+    }
+
+    /**
+     * Make a name tree node.
+     *
+     * @return the new name tree node
+     */
+    public PDFNameTreeNode makeNameTreeNode() {
+        PDFNameTreeNode node = new PDFNameTreeNode();
+        getDocument().registerObject(node);
+        return node;
+    }    
+    
     /* ========================= links ===================================== */
+    // Some of the "yoffset-only" functions in this part are obsolete and can
+    // possibly be removed or deprecated. Some are still called by PDFGraphics2D
+    // (although that could be changed, they don't need the yOffset param anyway).
+
+    /**
+     * Create a PDF link to an existing PDFAction object
+     *
+     * @param rect the hotspot position in absolute coordinates
+     * @param pdfAction the PDFAction that this link refers to
+     * @return the new PDFLink object, or null if either rect or pdfAction is null
+     */
+    public PDFLink makeLink(Rectangle2D rect, PDFAction pdfAction) {
+        if (rect == null || pdfAction == null) {
+            return null;
+        } else {
+            PDFLink link = new PDFLink(rect);
+            link.setAction(pdfAction);
+            getDocument().registerObject(link);
+            return link;
+            // does findLink make sense? I mean, how often will it happen that several
+            // links have the same target *and* the same hot rect? And findLink has to
+            // walk and compare the entire link list everytime you call it...
+        }
+    }
 
     /**
      * Make an internal link.
@@ -845,32 +956,10 @@ public class PDFFactory {
                             int linkType, float yoffset) {
 
         //PDFLink linkObject;
-        int index;
-
         PDFLink link = new PDFLink(rect);
 
         if (linkType == PDFLink.EXTERNAL) {
-            // check destination
-            if (destination.startsWith("http://")) {
-                PDFUri uri = new PDFUri(destination);
-                link.setAction(uri);
-            } else if (destination.endsWith(".pdf")) {    // FileSpec
-                PDFGoToRemote remote = getGoToPDFAction(destination, null, -1);
-                link.setAction(remote);
-            } else if ((index = destination.indexOf(".pdf#page=")) > 0) {
-                //String file = destination.substring(0, index + 4);
-                int page = Integer.parseInt(destination.substring(index + 10));
-                PDFGoToRemote remote = getGoToPDFAction(destination, null, page);
-                link.setAction(remote);
-            } else if ((index = destination.indexOf(".pdf#dest=")) > 0) {
-                //String file = destination.substring(0, index + 4);
-                String dest = destination.substring(index + 10);
-                PDFGoToRemote remote = getGoToPDFAction(destination, dest, -1);
-                link.setAction(remote);
-            } else {                               // URI
-                PDFUri uri = new PDFUri(destination);
-                link.setAction(uri);
-            }
+            link.setAction(getExternalAction(destination));
         } else {
             // linkType is internal
             String goToReference = getGoToReference(destination, yoffset);
@@ -888,11 +977,62 @@ public class PDFFactory {
         return link;
     }
 
-    private String getGoToReference(String destination, float yoffset) {
+    /**
+     * Create/find and return the appropriate external PDFAction according to the target
+     *
+     * @param target The external target. This may be a PDF file name
+     * (optionally with internal page number or destination) or any type of URI.
+     * @return the PDFAction thus created or found
+     */
+    public PDFAction getExternalAction(String target) {
+        int index;
+        String targetLo = target.toLowerCase();
+        // HTTP URL?
+        if (targetLo.startsWith("http://")) {
+            return new PDFUri(target);
+        // Bare PDF file name?
+        } else if (targetLo.endsWith(".pdf")) {
+            return getGoToPDFAction(target, null, -1);
+        // PDF file + page?
+        } else if ((index = targetLo.indexOf(".pdf#page=")) > 0) {
+            String filename = target.substring(0, index + 4);
+            int page = Integer.parseInt(target.substring(index + 10));
+            return getGoToPDFAction(filename, null, page);
+        // PDF file + destination?
+        } else if ((index = targetLo.indexOf(".pdf#dest=")) > 0) {
+            String filename = target.substring(0, index + 4);
+            String dest = target.substring(index + 10);
+            return getGoToPDFAction(filename, dest, -1);
+        // None of the above? Default to URI:
+        } else {
+            return new PDFUri(target);
+        }
+    }
+
+    /**
+     * Create or find a PDF GoTo with the given page reference string and Y offset,
+     * and return its PDF object reference
+     *
+     * @param pdfPageRef the PDF page reference, e.g. "23 0 R"
+     * @param yoffset the distance from the bottom of the page in points
+     * @return the GoTo's object reference
+     */
+    public String getGoToReference(String pdfPageRef, float yoffset) {
+        return getPDFGoTo(pdfPageRef, new Point2D.Float(0.0f, yoffset)).referencePDF();
+    }
+
+    /**
+     * Finds and returns a PDFGoTo to the given page and position.
+     * Creates the PDFGoTo if not found.
+     *
+     * @param pdfPageRef the PDF page reference
+     * @param position the (X,Y) position in points
+     *
+     * @return the new or existing PDFGoTo object
+     */
+    public PDFGoTo getPDFGoTo(String pdfPageRef, Point2D position) {
         getDocument().getProfile().verifyActionAllowed();
-        String goToReference = null;
-        PDFGoTo gt = new PDFGoTo(destination);
-        gt.setYPosition(yoffset);
+        PDFGoTo gt = new PDFGoTo(pdfPageRef, position);
         PDFGoTo oldgt = getDocument().findGoTo(gt);
         if (oldgt == null) {
             getDocument().assignObjectNumber(gt);
@@ -900,9 +1040,7 @@ public class PDFFactory {
         } else {
             gt = oldgt;
         }
-
-        goToReference = gt.referencePDF();
-        return goToReference;
+        return gt;
     }
 
     /**
@@ -943,6 +1081,42 @@ public class PDFFactory {
         return remote;
     }
 
+    /**
+     * Make an outline object and add it to the given parent
+     *
+     * @param parent the parent PDFOutline object (may be null)
+     * @param label the title for the new outline object
+     * @param actionRef the action reference string to be placed after the /A
+     * @param showSubItems whether to initially display child outline items
+     * @return the new PDF outline object
+     */
+    public PDFOutline makeOutline(PDFOutline parent, String label,
+                                  String actionRef, boolean showSubItems) {
+        PDFOutline pdfOutline = new PDFOutline(label, actionRef, showSubItems);
+        if (parent != null) {
+            parent.addOutline(pdfOutline);
+        }
+        getDocument().registerObject(pdfOutline);
+        return pdfOutline;
+    }
+
+    /**
+     * Make an outline object and add it to the given parent
+     *
+     * @param parent the parent PDFOutline object (may be null)
+     * @param label the title for the new outline object
+     * @param pdfAction the action that this outline item points to - must not be null!
+     * @param showSubItems whether to initially display child outline items
+     * @return the new PDFOutline object, or null if pdfAction is null
+     */
+    public PDFOutline makeOutline(PDFOutline parent, String label,
+                                  PDFAction pdfAction, boolean showSubItems) {
+        return pdfAction == null
+                 ? null
+                 : makeOutline(parent, label, pdfAction.getAction(), showSubItems);
+    }
+
+    // This one is obsolete now, at least it isn't called from anywhere inside FOP
     /**
      * Make an outline object and add it to the given outline
      *
@@ -958,17 +1132,10 @@ public class PDFFactory {
                                   boolean showSubItems) {
 
         String goToRef = getGoToReference(destination, yoffset);
-        PDFOutline obj = new PDFOutline(label, goToRef, showSubItems);
-
-        if (parent != null) {
-            parent.addOutline(obj);
-        }
-        getDocument().registerObject(obj);
-        return obj;
+        return makeOutline(parent, label, goToRef, showSubItems);
     }
 
 
-
     /* ========================= fonts ===================================== */
 
     /**
@@ -1024,15 +1191,12 @@ public class PDFFactory {
                  * cmap.addContents();
                  * this.objects.add(cmap);
                  */
-                font =
-                    (PDFFontNonBase14)PDFFont.createFont(fontname, fonttype,
-                                                         basefont,
-                                                         "Identity-H");
+                font = (PDFFontNonBase14)PDFFont.createFont(fontname, fonttype,
+                                                            basefont, "Identity-H");
             } else {
 
-                font =
-                    (PDFFontNonBase14)PDFFont.createFont(fontname, fonttype,
-                                                         basefont, encoding);
+                font = (PDFFontNonBase14)PDFFont.createFont(fontname, fonttype,
+                                                            basefont, encoding);
             }
             getDocument().registerObject(font);
 
@@ -1045,12 +1209,12 @@ public class PDFFactory {
                 } else {
                     cidMetrics = (CIDFont)metrics;
                 }
-                PDFCIDSystemInfo sysInfo =
-                    new PDFCIDSystemInfo(cidMetrics.getRegistry(),
+                PDFCIDSystemInfo sysInfo
+                    = new PDFCIDSystemInfo(cidMetrics.getRegistry(),
                                          cidMetrics.getOrdering(),
                                          cidMetrics.getSupplement());
-                PDFCIDFont cidFont =
-                    new PDFCIDFont(basefont,
+                PDFCIDFont cidFont
+                    = new PDFCIDFont(basefont,
                                    cidMetrics.getCIDType(),
                                    cidMetrics.getDefaultWidth(),
                                    getSubsetWidths(cidMetrics), sysInfo,
@@ -1153,9 +1317,9 @@ public class PDFFactory {
         int value = 0;
         for (int i = 0, c = cidSubset.length(); i < c; i++) {
             int shift = i % 8;
-            boolean b = cidSubset.get(i); 
+            boolean b = cidSubset.get(i);
             if (b) {
-                value |= 1 << 7 - shift; 
+                value |= 1 << 7 - shift;
             }
             if (shift == 7) {
                 baout.write(value);
@@ -1335,7 +1499,7 @@ public class PDFFactory {
     public PDFICCBasedColorSpace makeICCBasedColorSpace(PDFResourceContext res,
             String explicitName, PDFICCStream iccStream) {
         PDFICCBasedColorSpace cs = new PDFICCBasedColorSpace(explicitName, iccStream);
-        
+
         getDocument().registerObject(cs);
 
         if (res != null) {
@@ -1343,7 +1507,7 @@ public class PDFFactory {
         } else {
             getDocument().getResources().addColorSpace(cs);
         }
-        
+
         return cs;
     }
 
