@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSTextPainter.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: PSTextPainter.java 542237 2007-05-28 14:31:24Z jeremias $ */
 
 package org.apache.fop.render.ps;
 
@@ -66,7 +66,7 @@ import org.apache.fop.fonts.FontTriplet;
  * (todo) use drawString(AttributedCharacterIterator iterator...) for some
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PSTextPainter.java 426576 2006-07-28 15:44:37Z jeremias $
+ * @version $Id: PSTextPainter.java 542237 2007-05-28 14:31:24Z jeremias $
  */
 public class PSTextPainter implements TextPainter {
     
@@ -367,8 +367,8 @@ public class PSTextPainter implements TextPainter {
     private int getWeight(AttributedCharacterIterator aci) {
         Float taWeight = (Float)aci.getAttribute(TextAttribute.WEIGHT);
         return ((taWeight != null) &&  (taWeight.floatValue() > 1.0)) 
-                       ? Font.BOLD
-                       : Font.NORMAL;
+                       ? Font.WEIGHT_BOLD
+                       : Font.WEIGHT_NORMAL;
     }
 
     private Font makeFont(AttributedCharacterIterator aci) {
@@ -402,7 +402,7 @@ public class PSTextPainter implements TextPainter {
                 }
             }
         }
-        FontTriplet triplet = fontInfo.fontLookup("any", style, Font.NORMAL);
+        FontTriplet triplet = fontInfo.fontLookup("any", style, Font.WEIGHT_NORMAL);
         int fsize = (int)(fontSize.floatValue() * 1000);
         return fontInfo.getFontInstance(triplet, fsize);
     }
@@ -411,7 +411,7 @@ public class PSTextPainter implements TextPainter {
         final String style = getStyle(aci);
         final int weight = getWeight(aci);
         int fStyle = java.awt.Font.PLAIN;
-        if (weight == Font.BOLD) {
+        if (weight == Font.WEIGHT_BOLD) {
             fStyle |= java.awt.Font.BOLD;
         }
         if ("italic".equals(style)) {
