@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TXTRenderer.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: TXTRenderer.java 542237 2007-05-28 14:31:24Z jeremias $ */
  
 package org.apache.fop.render.txt;
 
@@ -27,8 +27,6 @@ import java.io.OutputStream;
 import java.util.List;
 import java.util.Map;
 
-import org.apache.avalon.framework.configuration.Configuration;
-import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.area.Area;
 import org.apache.fop.area.CTM;
@@ -115,12 +113,6 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     public String getMimeType() {
         return "text/plain";
     }
-
-    /** @see org.apache.fop.render.AbstractRenderer */
-    public void configure(Configuration conf) throws ConfigurationException {
-        super.configure(conf);
-        this.encoding = conf.getChild("encoding", true).getValue(null);
-    }
     
     /**
      * Sets the encoding of the target file.
