@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFGraphics2D.java 474225 2006-11-13 10:08:19Z jeremias $ */
+/* $Id: PDFGraphics2D.java 542237 2007-05-28 14:31:24Z jeremias $ */
 
 package org.apache.fop.svg;
 
@@ -100,7 +100,7 @@ import java.util.List;
  * This takes a pdf document and draws into it.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PDFGraphics2D.java 474225 2006-11-13 10:08:19Z jeremias $
+ * @version $Id: PDFGraphics2D.java 542237 2007-05-28 14:31:24Z jeremias $
  * @see org.apache.batik.ext.awt.g2d.AbstractGraphics2D
  */
 public class PDFGraphics2D extends AbstractGraphics2D {
@@ -1429,7 +1429,7 @@ public class PDFGraphics2D extends AbstractGraphics2D {
             }
             float siz = gFont.getSize2D();
             String style = gFont.isItalic() ? "italic" : "normal";
-            int weight = gFont.isBold() ? Font.BOLD : Font.NORMAL;
+            int weight = gFont.isBold() ? Font.WEIGHT_BOLD : Font.WEIGHT_NORMAL;
             FontTriplet triplet = fontInfo.fontLookup(n, style, weight);
             fontState = fontInfo.getFontInstance(triplet, (int)(siz * 1000 + 0.5));
         } else {
